#!/usr/bin/perl -w
#
# SSH Plugin 
# Jeremy Norris
#
# Copyright 2000 Rebel.com Inc. All Rights Reserved.

use CGI;
use Config::TextKV;
use strict;
push @INC, "/home/httpd/cgi-bin/netwinder";
require 'common.pl';
require 'layout.pl';

# clean up path and environment variables for safety.
$ENV{'PATH'} = '/bin:/usr/bin';
$ENV{'SHELL'} = '/bin/sh';
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

# Access control
my $auid = get_admin_name() || 'httpd';
my $ruid = CGI::remote_user() || '';
if($ruid ne $auid) {
	print start_gui("Error",'user','uniquemonkey',7).
	h3("An error has occurred").
	CGI::p("<i>This module requires Administrator privileges.</i>").
	return_link('/cgi-bin/netwinder/options.cgi').end_gui();
	exit 0;
}

# set configuration options and defaults.
my %options = (
  PermitRootLogin => 'yes',
  X11Forwarding => 'yes',
  Protocol => '2',
  PasswordAuthentication => 'yes',
  KeepAlive => 'yes',
);

my $CONFIG_FILE = '/etc/ssh/sshd_config';
my $PID = '/var/run/sshd.pid';

my $default_desc = 'Current SSH Status';
my %mode = (
  list =>      [\&list, $default_desc],
  configure => [\&configure, 'Edit SSH Configuration'],
  save =>      [\&save, $default_desc],
  start =>     [\&action, $default_desc, 'start'],
  stop =>      [\&action, $default_desc, 'stop'],
  restart =>   [\&action, $default_desc, 'restart'],
);

my $m = CGI::param('mode') || 'list';
$m = 'list' unless exists $mode{$m};

print 
  start_gui($mode{$m}[1], 'admin', 'uniquemonkey', 7) .
  h3($mode{$m}[1]);
my $result = &{$mode{$m}[0]}($mode{$m}[2]); # call named subroutine
print end_gui();
exit;

sub list {

  my($status, $action);

  # generate action list.
  if(-e $PID) {
    $status = 'SSH server is running.'; 
    $action = 
      CGI::a({-href=>'ssh.cgi?mode=restart'}, 'Restart') . " " .
      CGI::a({-href=>'ssh.cgi?mode=stop'}, 'Stop');
  }
  else {
    $status = 'SSH server is not running.'; 
    $action =
      CGI::a({-href=>'ssh.cgi?mode=start'}, 'Start');
  }      
  $action .= " " . CGI::a({-href=>'ssh.cgi?mode=configure'}, 'Configure');

  print
    start_table() .
    Tr(th(['Status', 'Action'])) . 
    Tr(td([$status, $action])) .
    end_table();

}

sub configure {

  # initialize/parse configuration file.
  my $config = Config::TextKV->new($CONFIG_FILE, ' ', '#');

  # set options descriptions, and orders.
  my %option_descriptions = (
    PermitRootLogin => 'Permit Root Login',
    X11Forwarding => 'X11 Forwarding',
    Protocol => 'SSH Protocol',
    PasswordAuthentication => 'Password Authentication',
    KeepAlive => 'Keep Alive',
  ); 
  my %order = (
    3 => 'PermitRootLogin',
    4 => 'X11Forwarding',
    1 => 'Protocol',
    2 => 'PasswordAuthentication',
    5 => 'KeepAlive',
  );

  # configuration options translation hashes.
  my %t = (
    PermitRootLogin => {
      yes => 'Yes',
      qw(without-password) => 'Without Password',
      no => 'No'
    }, 
    X11Forwarding => {
      yes => 'Yes',
      no => 'No'
    },
    Protocol => {
      1 => 'SSH1',
      2 => 'SSH2',
      '2,1', 'Both',
    },
    PasswordAuthentication => {
      yes => 'Yes',
      no => 'No'
    },
    KeepAlive => {
      yes => 'Yes',
      no => 'No'
    },
  );  

  # build table data
  my($opt, $default);
  my $table_data = '';
  foreach $opt (sort keys %order) {
    $opt = $order{$opt};
    my @values = keys %{$t{$opt}};
    if(!defined($default = $config->GetKey($opt))) {
      $default = $options{$opt};
    }
    # ensure we have a translation for $default.
    if(!exists($t{$opt}{$default})) {
      $default = $options{$opt};
    }
    $table_data .= Tr(
      te($option_descriptions{$opt}) .
      te(CGI::radio_group(
        -name=>$opt, -value=>\@values,
        -labels=>\%{$t{$opt}}, -default=>$default)));
  }

  print 
    CGI::start_form(-action=>'ssh.cgi', -method=>'post') .
    start_table() .
    Tr(th({colspan=>2},'&nbsp;')) .
    $table_data .
    CGI::delete('mode') . 
    CGI::hidden(-name=>'mode', default=>'save') .
    end_table() .
    save_cancel_buttons('ssh.cgi') .
    CGI::end_form();

}

sub save {

  # initialize/parse configuration file.
  my $config = Config::TextKV->new($CONFIG_FILE, ' ', '#');

  # save settings
  my $key;
  # strip any whitespace from cgi parms.
  my @temp = keys %options;
  foreach $key (keys %options) {
    $options{$key} = CGI::param($key);
    # use 'force' so that if the desired options do
    # not exist, in the configuration file, they
    # will be added with the default values specified
    # in the %options hash.
    $config->SetKey($key, $options{$key}, 'force');
  }

  my $message = 'Configuration changes have been saved';
  # Send a hangup to sshd if it is running.
  if(-e $PID) {
    if(!system('kill', '-HUP', substr(`/bin/cat $PID`, 0, -1))) {
      $message .= ' and the SSH server has been restarted';
    }
  }

  print CGI::p($message . ".") . CGI::p();

  list;

}

# called with either 'start', 'stop' or 'restart' as the argument.
sub action {

  my $action = shift;

  # set the real user ID (uid) of this process.
  $<=0;

  if(!run_script('sshd', $action)) {
    if($action eq 'start') {
      system('/sbin/chkconfig --add sshd');
    }
    elsif($action eq 'stop') {
      system('/sbin/chkconfig --del sshd');
    }
    print CGI::p("Service $action successful.");
  }
  else {
    print CGI::p("Unable to $action SSH server.");
  }
 
  list;

}
